(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('ramda'), require('@angular/core'), require('@angular/router')) :
	typeof define === 'function' && define.amd ? define(['exports', 'ramda', '@angular/core', '@angular/router'], factory) :
	(factory((global.district01 = global.district01 || {}, global.district01['ng-redux-router'] = {}),global.ramda,global.ng.core,global.ng.router));
}(this, (function (exports,ramda,core,router) { 'use strict';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var ROUTE_PENDING = "ROUTE_PENDING";
var ROUTE_CANCEL = "ROUTE_CANCEL";
var ROUTE_UPDATE = "ROUTE_UPDATE";
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var rebuildStateTree = function (rootState, key) {
    if (!rootState) {
        return {};
    }
    if (rootState.firstChild) {
        return Object.assign({}, rootState[key], rebuildStateTree(rootState.firstChild, key));
    }
    return Object.assign({}, rootState[key]);
};
var getPath = function (url) {
    if (url === void 0) { url = ""; }
    return url.split(/[?#]/)[0];
};
var getFragment = function (url) {
    if (url === void 0) { url = ""; }
    var /** @type {?} */ segments = ramda.pathOr(url, ["url"], url).split("/").pop().split(/[#]/);
    return segments.length > 1 ? segments[1] : "";
};
var getData = function (rootState) {
    if (!rootState) {
        return null;
    }
    if (rootState.firstChild) {
        return getData(rootState.firstChild);
    }
    return ramda.clone(rootState.data);
};
var routesAreEqual = function (current, next) {
    var /** @type {?} */ getProp = function (prop, obj) { return ramda.path([prop], obj); };
    var /** @type {?} */ equalProp = function (prop) {
        var objs = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            objs[_i - 1] = arguments[_i];
        }
        return ramda.equals.apply(null, objs.map(function (obj) { return getProp(prop, obj); }));
    };
    return (equalProp("url", current, next) &&
        equalProp("params", current, next) &&
        equalProp("query", current, next) &&
        equalProp("fragment", current, next));
};
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var initRouteChange = function (route) {
    return {
        type: ROUTE_PENDING,
        route: {
            url: route.urlAfterRedirects || route.url,
            path: getPath(route.urlAfterRedirects || route.url),
            params: rebuildStateTree(route.state.root, "params"),
            query: rebuildStateTree(route.state.root, "queryParams"),
            fragment: getFragment(route.url),
            data: getData(route.state.root),
        },
    };
};
var cancelRouteChange = function (route) {
    return {
        type: ROUTE_CANCEL,
    };
};
var endRouteChange = function (route) {
    return {
        type: ROUTE_UPDATE,
        route: route,
    };
};
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var ReduxRouter = /** @class */ (function () {
    /**
     * @param {?} router
     */
    function ReduxRouter(router$$1) {
        this.router = router$$1;
    }
    /**
     * @param {?} dispatch
     * @return {?}
     */
    ReduxRouter.prototype.initialize = function (dispatch) {
        if (!dispatch) {
            return console.error("No dispatcher provided."); // tslint:disable-line:no-console
        }
        this.routerSubscription = this.router.events
            .subscribe(function (routeEvent) {
            if (routeEvent instanceof router.RoutesRecognized) {
                return dispatch(initRouteChange(routeEvent));
            }
            if (routeEvent instanceof router.NavigationCancel) {
                return dispatch(cancelRouteChange(routeEvent));
            }
            if (routeEvent instanceof router.NavigationEnd) {
                return dispatch(endRouteChange(routeEvent));
            }
        });
    };
    return ReduxRouter;
}());
ReduxRouter.decorators = [
    { type: core.Injectable },
];
/** @nocollapse */
ReduxRouter.ctorParameters = function () { return [
    { type: router.Router, },
]; };
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var REDUX_ROUTER_INITIAL_STATE = {
    current: null,
    previous: null,
    pending: null,
};
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var routerReducer = function (state, action) {
    if (state === void 0) { state = REDUX_ROUTER_INITIAL_STATE; }
    if (action.type === ROUTE_PENDING) {
        if (!routesAreEqual(state.current, action.route)) {
            return Object.assign({}, state, { pending: ramda.clone(action.route) });
        }
    }
    if (action.type === ROUTE_CANCEL) {
        return Object.assign({}, state, { pending: null });
    }
    if (action.type === ROUTE_UPDATE) {
        return {
            current: ramda.clone(state.pending),
            previous: ramda.clone(state.current),
            pending: null,
        };
    }
    return state;
};
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var ReduxRouterModule = /** @class */ (function () {
    function ReduxRouterModule() {
    }
    return ReduxRouterModule;
}());
ReduxRouterModule.decorators = [
    { type: core.NgModule, args: [{
                imports: [
                    router.RouterModule,
                ],
                providers: [
                    ReduxRouter,
                ],
            },] },
];
/** @nocollapse */
ReduxRouterModule.ctorParameters = function () { return []; };

exports.ReduxRouter = ReduxRouter;
exports.routerReducer = routerReducer;
exports.ROUTE_PENDING = ROUTE_PENDING;
exports.ROUTE_CANCEL = ROUTE_CANCEL;
exports.ROUTE_UPDATE = ROUTE_UPDATE;
exports.REDUX_ROUTER_INITIAL_STATE = REDUX_ROUTER_INITIAL_STATE;
exports.ReduxRouterModule = ReduxRouterModule;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=district01-ng-redux-router.umd.js.map
